#  tests for pykakasi-2.0.4-py36ha15d459_1 (this is a generated file);
print('===== testing package: pykakasi-2.0.4-py36ha15d459_1 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
# the upstream session-scoped test fixture redundantly re-generates
# the dictionaries that are already packaged, see
# https://github.com/miurahr/pykakasi/blob/v2.0.4/tests/conftest.py
# unless we specify the following environment variable:
import os
os.environ["TOX_ENV_DIR"] = "dummy"

import pytest
pytest.main(["tests/"])
#  --- run_test.py (end) ---

print('===== pykakasi-2.0.4-py36ha15d459_1 OK =====');
print("import: 'pykakasi'")
import pykakasi

